#!/usr/bin/env python3
import argparse
import json
import logging
import sys
from avi.sdk.avi_api import ApiSession
from avi.sdk.saml_avi_api import OktaSAMLApiSession, OneloginSAMLApiSession
from avi.sdk.utils.api_utils import ApiUtils
from requests.packages import urllib3

logger = logging.getLogger(__name__)
ch = logging.StreamHandler(sys.stdout)
root_logger = logging.getLogger()
root_logger.setLevel(logging.DEBUG)
root_logger.addHandler(ch)

urllib3.disable_warnings()


class SAMLExample(object):
    """
    Provides example of following operations
    1. Create basic vs
    2. Create vs with SSL objects
    3. Create vs with custom parameters like application profile
    4. Inventory API calls for virtualservice
    5. Metrics API calls for virtualservice metrics
    """
    def __init__(self, api):
        self.api = api
        self.api_utils = ApiUtils(api)

    @staticmethod
    def get_idp_class(idp):
        """
        This return corresponding idp class.
        :param idp: idp type such as okta, onelogin, pingfed
        :return: IDP class or ApiSession class
        """

        if str(idp).lower() == "oktasamlapisession":
            print("Using OktaSAMLApiSession class")
            idp_class = OktaSAMLApiSession
        elif str(idp).lower() == 'oneloginsamlapisession':
            print("Using OktaSAMLApiSession class")
            idp_class = OneloginSAMLApiSession
        else:
            print("Please provide correct IDP class. Supported classes are "
                  "OktaSAMLApiSession, OneloginApiSession.")
            idp_class = None
        return idp_class

    def get_server_obj(self, servers):
        """
        :param servers Input pool servers.
        """
        servers_obj = []
        for server in servers:
            parts = server.split(':')
            ip_addr = parts[0]
            port = parts[1] if len(parts) == 2 else 80
            servers_obj.append({
                'ip': {
                    'addr': ip_addr,
                    'type': 'V4'
                },
                'port': port
            })
        return servers_obj

    def create_basic_vs(self, vs_name, vip, servers=None):
        # first create pool for virtual service
        servers_obj = self.get_server_obj(servers)
        pool_name = vs_name + '-pool'
        pool_obj = self.create_pool(pool_name, servers_obj)
        pool_ref = self.api.get_obj_ref(pool_obj)

        # create virtual service dict
        services_obj = [{'port': 80, 'enable_ssl': False}]
        vs_obj = {
            'name': vs_name,
            'type': 'VS_TYPE_NORMAL',
            'ip_address': {
                'addr': vip,
                'type': 'V4'
            },
            'enabled': True,
            'services': services_obj,
            'application_profile_name': 'System-HTTP',
            'pool_ref': pool_ref
        }

        # post VS data in json format to avi api
        resp = self.api.post('virtualservice', data=json.dumps(vs_obj))
        if resp.status_code in range(200, 299):
            logger.debug('Virtual service created successfully %s' % vs_name)
        else:
            logger.debug('Error creating virtual service : %s' % resp.text)
        return

    def create_pool(self, name, servers_obj, monitor_names=None,
                    lb_algorithm='LB_ALGORITHM_LEAST_CONNECTIONS'):
        health_monitors = None
        if monitor_names:
            health_monitors = []
            for monitor_name in monitor_names:
                health_monitor_tcp = self.api.get_object_by_name('healthmonitor',
                                                                 monitor_name)
                health_monitors.append(self.api.get_obj_ref(health_monitor_tcp))
        print(health_monitors)
        pool_name = name
        pool_obj = {
            "lb_algorithm": lb_algorithm,
            "default_server_port": 80,
            "name": pool_name,
            "servers": servers_obj,
            'health_monitor_refs': health_monitors
        }
        resp = self.api.post('pool', data=json.dumps(pool_obj))
        if resp.status_code in range(200, 299):
            return resp
        else:
            print('Error in creating pool :%s' % resp.text)
            exit(0)
        return


if __name__ == '__main__':

    parser = argparse.ArgumentParser()
    parser.add_argument('-idp', '--idp_class', help="IDP class such as OktaSAMLApiSession, "
                                                    "OneloginSAMLApiSession, etc",
                        default="OktaSAMLApiSession")
    parser.add_argument(
        '-s', '--server_ips',
        help='Pool Server IPs comma separated Eg. 1.1.1.1,2.2.2.2',
        default='1.1.1.1,1.1.1.2')
    parser.add_argument('-u', '--idp_user', help='IDP username',
                        default='foo@avinetworks.com')
    parser.add_argument('-p', '--idp_password', help='IDP user password',
                        default='foo123')
    parser.add_argument('-t', '--tenant', help='tenant name',
                        default=None)
    parser.add_argument('--tenant-uuid', help='tenant uuid',
                        default=None)
    parser.add_argument('-c', '--controller_ip', help='controller ip')
    parser.add_argument('-i', '--vip', help='VIP address')

    args = parser.parse_args()
    print('parsed args', args)
    idp_cls = SAMLExample.get_idp_class(args.idp_class)

    # SAML authentication with given IDP and get the controller session
    api = ApiSession.get_session(args.controller_ip, args.idp_user, args.idp_password,
                                 tenant=args.tenant, tenant_uuid=args.tenant_uuid, idp_class=idp_cls)

    servers = [server.strip() for server in args.server_ips.split(',')]
    vse = SAMLExample(api)

    # Create basic VS
    vip = args.vip if args.vip else '10.10.42.2'
    vse.create_basic_vs('basic-vs', vip, servers)



'''
Created on Feb 10, 2016

@author: grastogi
'''
import json
import logging

log = logging.getLogger(__name__)


class ApiUtils(object):
    '''
    Common utilities for the Avi APIs
    '''
    def __init__(self, api_session):
        '''
        Constructor
        :param api_session: ApiSession
        '''
        self.api = api_session

    def import_ssl_certificate(self, name, key, cert, cert_type='', key_passphrase='',
                               tenant='', tenant_uuid=''):
        """import_ssl_certificate takes cert name server key, cert, cert_type, passphrase
        returns session's response object

        Starting from 16.3, cert_type is required.
        API defaults to SSL_CERTIFICATE_TYPE_VIRTUALSERVICE if not provided"""
        if not cert_type:
            cert_type = 'SSL_CERTIFICATE_TYPE_VIRTUALSERVICE'
        ssl_kc_obj = {
            'name': name,
            'key': key,
            'type': cert_type,
            'certificate': {
                'certificate': cert
                },
            'key_passphrase': key_passphrase
        }
        resp = self.api.post(
                path='sslkeyandcertificate',
                data=json.dumps(ssl_kc_obj),
                tenant=tenant, tenant_uuid=tenant_uuid)
        return resp

    def get_metrics(
            self, entity_type, entity_name, entity_uuid='', metric_id='',
            step=300, limit=1, start='', stop='', tenant='admin',
            tenant_uuid='', **query_options):
        """
        This is utility method to fetch metrics from the controller.
        Eg.
        1. api_utils.get_metrics('virtualservice', 'vs1',
            metric_id='l4_client.avg_bandwidth')
            This gets 1 vs1's network bandwidth metrics.
        2.
        :param entity_type: 'virtualservice, serviceengine, pool, tenant'
        :param entity_name: string corresponding to the object
        :param entity_uuid: uuid of the entity. If entity_uuid and entity_name
            are blank then it is treated as collection API where metrics
            for all the objects in the tenant would be fetched
        :param metric_id: comma separated string of metrics or list of
            metric_ids
        :param step: granularity of the metrics
        :param limit: number of data points for the metrics
        :param start: ISO8901 compatible start time for the metrics. Controller
            will adjust the start time based on the Stop time if not provided
        :param stop: ISO8901 compatible start time for the metrics. Controller
            sets stop as utc.now if not provided in the API
        :tenant: name of the tenant
        :tenant_uuid: uuid of the tenant
        :query_options: All the query_options are sent as the query parameters
            for the API call as per the Avi API Guide.
        """
        if entity_type == 'controller':
            path = 'analytics/metrics/%s' % entity_type
        else:
            if entity_name and not entity_uuid:
                resp = self.api.get_object_by_name(entity_type, entity_name)
                entity_uuid = self.api.get_obj_uuid(resp)
            path = 'analytics/metrics/%s/%s' % (entity_type, entity_uuid)
        if type(metric_id) == list:
            metric_id = ','.join(metric_id)
        if metric_id:
            query_options['metric_id'] = metric_id
        query_options['step'] = step
        query_options['limit'] = limit
        if start:
            query_options['start'] = start
        if stop:
            query_options['stop'] = stop
        rsp = self.api.get(path, tenant=tenant, tenant_uuid=tenant_uuid,
                           params=query_options)
        return rsp.json()

    def get_healthscore(
            self, entity_type, entity_name='', entity_uuid='',
            step=300, limit=1, start='', stop='', tenant='admin',
            tenant_uuid='', **query_options):
        """
        This is utility method to fetch healthscore from the controller.
        Eg.
        1. api_utils.get_metrics('virtualservice', 'vs1',
            metric_id='l4_client.avg_bandwidth')
            This gets 1 vs1's network bandwidth metrics.
        :param entity_type: 'virtualservice, serviceengine, pool, tenant'
        :param entity_name: string corresponding to the object
        :param entity_uuid: uuid of the entity. If entity_uuid and entity_name
            are blank then it is treated as collection API where metrics
            for all the objects in the tenant would be fetched
        :param step: granularity of the metrics
        :param limit: number of data points for the metrics
        :param start: ISO8901 compatible start time for the metrics. Controller
            will adjust the start time based on the Stop time if not provided
        :param stop: ISO8901 compatible start time for the metrics. Controller
            sets stop as utc.now if not provided in the API
        :tenant: name of the tenant
        :tenant_uuid: uuid of the tenant
        :query_options: All the query_options are sent as the query parameters
            for the API call as per the Avi API Guide.
        """
        if entity_name and not entity_uuid:
            resp = self.api.get_object_by_name(entity_type, entity_name)
            entity_uuid = self.api.get_obj_uuid(resp)
        path = 'analytics/metrics/%s/%s' % (entity_type, entity_uuid)
        query_options['step'] = step
        query_options['limit'] = limit
        if start:
            query_options['start'] = start
        if stop:
            query_options['stop'] = stop
        rsp = self.api.get(path, tenant=tenant, tenant_uuid=tenant_uuid,
                           params=query_options)
        return rsp.json()

    def get_metrics_collection(
            self, tenant='admin', tenant_uuid='', metric_requests=None,
            **query_options):
        """
        This is utility method to fetch metrics from the controller.
        Eg.
            {"metric_requests":[
                {"step":300,"limit":72,"id":"end_to_end",
                "entity_uuid":"virtualservice-76146076-be4f-4ccd-8253-68f2fb83f498",
                "metric_id":"l4_client.avg_total_rtt,l4_server.avg_total_rtt},
                {"step":300,"limit":72,"id":"l4_client.avg_bandwidth",
                "entity_uuid":"virtualservice-76146076-be4f-4ccd-8253-68f2fb83f498",
                "metric_id":"l4_client.avg_bandwidth"}
                ]
            }
        :param metric_requests: Array of metric_requests
        :tenant: name of the tenant
        :tenant_uuid: uuid of the tenant
        :query_options: All the query_options are sent as the query parameters
            for the API call as per the Avi API Guide.
        """
        if not metric_requests or type(metric_requests) != list:
            raise Exception(
                'No queries passed or invalid reqs %s' % metric_requests)
        data = {'metric_requests': metric_requests}
        path = 'analytics/metrics/collection'
        rsp = self.api.post(path, data=data, tenant=tenant,
                            tenant_uuid=tenant_uuid, params=query_options)
        return rsp.json()



#!/usr/bin/python
import os
import sys
import json
from avi.sdk.avi_api import ApiSession
from requests.packages import urllib3

urllib3.disable_warnings()

def switch_priorities(session, pool_uuid, pool_name, retries=5):
    if retries <= 0:
        return 'Too many retry attempts - aborting!'
    query = 'refers_to=pool:%s' % pool_uuid
    pg_result = session.get('poolgroup', params=query)
    if pg_result.count() == 0:
        return 'No pool group found referencing pool %s' % pool_name

    pg_obj = pg_result.json()['results'][0]

    highest_up_pool = None
    highest_down_pool = None

    for member in pg_obj['members']:
        priority_label = member['priority_label']
        member_ref = member['pool_ref']
        pool_runtime_url = ('%s/runtime/detail' %
                            member_ref.split('/api/')[1])
        pool_obj = session.get(pool_runtime_url).json()[0]
        if pool_obj['oper_status']['state'] == 'OPER_UP':
            if (not highest_up_pool or
                int(highest_up_pool[1]) < int(priority_label)):
                highest_up_pool = (member, priority_label,
                                   pool_obj['name'])
        elif (not highest_down_pool or
              int(highest_down_pool[1]) < int(priority_label)):
            highest_down_pool = (member, priority_label,
                                 pool_obj['name'])

    if not highest_up_pool:
        return ('No action required as all pools in the '
                'pool group are now down.')
    elif not highest_down_pool:
        return ('No action required as all pools in the '
                'pool group are now up.')

    if int(highest_down_pool[1]) <= int(highest_up_pool[1]):
        return ('No action required. The highest-priority available '
                'pool (%s) already has a higher priority than the '
                'highest-priority non-available pool (%s)' %
                (highest_up_pool[2], highest_down_pool[2]))

    highest_up_pool[0]['priority_label'] = highest_down_pool[1]
    highest_down_pool[0]['priority_label'] = highest_up_pool[1]

    p_result = session.put('poolgroup/%s' % pg_obj['uuid'], pg_obj)
    if p_result.status_code < 300:
        return ', '.join(['Pool %s priority changed to %s' % (p[0], p[1])
                          for p in ((highest_up_pool[2],
                                     highest_down_pool[1]),
                                    (highest_down_pool[2],
                                     highest_up_pool[1]))])
    if p_result.status_code == 412:
        return switch_priorities(session, pool_uuid,
                                 pool_name, retries - 1)

    return 'Error setting pool priority: %s' % p_result.text

def sticky_pool_group():
    token = os.environ.get('API_TOKEN')
    user = os.environ.get('USER')
    tenant = os.environ.get('TENANT')
    alert_dict = json.loads(sys.argv[1])
    events = alert_dict.get('events', [])
    if len(events) > 0:
        pool_uuid = events[0]['obj_uuid']
        pool_name = events[0]['obj_name']
        try:
            with ApiSession("localhost", user,
                            token=token,
                            tenant=tenant) as session:
                result = switch_priorities(session, pool_uuid, pool_name)
        except Exception as e:
            result = str(e)
    else:
        result = 'No event data for ControlScript'

    return result

# Use with a ControlScript and Alert(s) to perform 'sticky' failover of pool groups.
#
# Alert should trigger on 'Pool Up' and 'Pool Down' events.
#
# Create a ControlScript as follows:
#
# #!/usr/bin/python
#
# from avi.sdk.samples.sticky_pool_group import sticky_pool_group
#
# if __name__ == '__main__':
#   print sticky_pool_group()
#



import os
from setuptools import setup, find_packages
"""
This setup needs to be copied to the top level python directory and then
create the package.
"""
AVI_PIP_VERSION = '20.1.3b2'

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='avisdk',
    version=AVI_PIP_VERSION,
    packages=find_packages(exclude=['*migrationtools*',
                                    '*sdk.samples.autoscale*',
                                    '*sdk.test*'
                                   ]),
    description='Avi python API SDK for Avi Controller REST API with samples'
                ' and, utilities',
    url='https://github.com/avinetworks/sdk',
    author='Avi Networks',
    author_email='avisdk@avinetworks.com',
    scripts=['avi/sdk/samples/virtualservice_examples_api.py'],
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
    keywords='AVI ADC Loadbalancer automation datacenter SDK',
    license='Avi Networks',
    include_package_data=True,
    install_requires=['requests'],
    package_data={'avi': ['*.cfg', '*.conf', '*.crt', '*.crl', '*.json',
                          '*.key', '*.pem', '*.xml', '*.yaml', '*.rst']},
)
